/* Based on https://github.com/mcasimir/generator-mobileangularui */
/*========================================
=            Requiring stuffs            =
========================================*/

var gulp = require('gulp'),
	gulpif = require('gulp-if'),
	seq = require('run-sequence'),
	connect = require('gulp-connect'),
	less = require('gulp-less'),
	uglify = require('gulp-uglify'),
	sourcemaps = require('gulp-sourcemaps'),
	cssmin = require('gulp-cssmin'),
	concat = require('gulp-concat'),
	rimraf = require('gulp-rimraf'),
	mobilizer = require('gulp-mobilizer'),
	replace = require('gulp-replace'),
	streamqueue = require('streamqueue'),
	rename = require('gulp-rename'),
	path = require('path'),
	source = require('vinyl-source-stream'),
	buffer = require('vinyl-buffer'),
	browserify = require('browserify'),
	babelify = require('babelify');

/*=====================================
=        Configuration                =
=====================================*/

var config = {
	src: __dirname + '/src',
	dest: './www',
	app: './src/js/index.js',
	assets: '/assets',
	brand: '<%=name%>',
	minify_images: true,

		vendor: {
		js: [],
		fonts: []
	},

	server: {
		host: '0.0.0.0',
		port: '3000'
	},

	weinre: {
		httpPort: 8001,
		boundHost: 'localhost',
		verbose: false,
		debug: false,
		readTimeout: 5,
		deathTimeout: 15
	}
};

config.assets = path.join(config.dest, config.assets);
config.vendor.js.push('/js/vendor/**/*');

/*-----  End of Configuration  ------*/


var panic = function(err) {

	//throw err;
	console.log(err);
	this.emit('end');

};


/*================================================
=            Report Errors to Console            =
================================================*/

gulp.on('error', function(e) {
	console.log(e.err.stack);
	this.emit('end');
});


/*=========================================
=            Clean dest folder            =
=========================================*/

gulp.task('clean', function() {
	return gulp.src([
			path.join(config.dest, 'index.html'),
			path.join(config.dest, 'images'),
			path.join(config.dest, 'vendor'),
			path.join(config.dest, 'css'),
			path.join(config.dest, 'js'),
			path.join(config.dest, 'fonts'),
			config.assets
		], {
			read: false
		})
		.pipe(rimraf());
});


/*==========================================
=            Start a web server            =
==========================================*/

gulp.task('connect', function() {
	if (typeof config.server === 'object') {
		connect.server({
			root: config.dest,
			host: config.server.host,
			port: config.server.port,
			livereload: true
		});
	} else {
		throw new Error('Connect is not configured');
	}
});


/*==============================================================
=            Setup live reloading on source changes            =
==============================================================*/

gulp.task('livereload', function() {
	gulp.src(path.join(config.dest, '*.html'))
		.pipe(connect.reload());
});

/*==================================
=            Copy images           =
==================================*/

gulp.task('images', function() {
	return gulp.src(config.src + '/images/**/*')
		.pipe(gulp.dest(path.join(config.assets, 'images')));
});

/*==================================
=            Copy vendor           =
==================================*/

gulp.task('vendor', function() {
	return gulp.src(config.src + '/vendor/**/*')
		.pipe(gulp.dest(path.join(config.assets, 'vendor')));
});
/*==================================
=            Copy fonts            =
==================================*/

gulp.task('fonts', function() {
	return gulp.src(config.src + '/fonts/**/*')
		.pipe(gulp.dest(path.join(config.assets, 'fonts')));
});

/*==================================
=            Copy CSS              =
==================================*/

gulp.task('css', function() {
	return gulp.src(config.src + '/css/**/*.css')
		.pipe(gulp.dest(path.join(config.assets, 'css')));
});


/*=================================================
=            Copy html files to dest              =
=================================================*/

gulp.task('html', function() {
	var inject = [];


	if (typeof config.weinre === 'object') {
		inject.push('<script src="http://' + config.weinre.boundHost + ':' + config.weinre.httpPort + '/target/target-script-min.js"></script>');
	}
	if (config.cordova) {
		inject.push('<script src="cordova.js"></script>');
	}
	gulp.src([config.src + '/html/**/*.html'])
		.pipe(replace('<!-- inject:js -->', inject.join('\n    ')))
		.pipe(gulp.dest(config.dest));
});

/*======================================================================
=            Compile, minify, mobilize less                            =
======================================================================*/

gulp.task('less', function() {

	gulp.src([config.src + '/src/less/index.less'])
		.pipe(less({
			paths: [path.resolve(__dirname, config.src + 'less'), path.resolve(__dirname, 'bower_components')]
		}))
		.pipe(mobilizer('app.css', {
			'app.css': {
				hover: 'exclude',
				screens: ['0px']
			}
		}))
		.pipe(cssmin())
		.pipe(rename({
			suffix: '.min'
		}))
		.pipe(gulp.dest(path.join(config.assets, 'css')));
});


/*====================================================================
=            Compile and minify js generating source maps            =
====================================================================*/
gulp.task('js', function() {

	var b = browserify(config.app, {
		debug: config.development,
		fullPaths: false,
		baseDir: './',
		cache: {},
		packageCache: {},
		transform: [babelify]
	});

	streamqueue({
				objectMode: true
			},
			gulp.src(config.vendor.js),
			b.bundle().pipe(source(config.app))
		)
		.on('error', panic)
		.pipe(buffer())
		.pipe(sourcemaps.init())
		.pipe(concat(config.brand + '.js'))
		.pipe(gulpif((process.env.NODE_ENV === 'production'), uglify()))
		.pipe(rename({
			suffix: '.min'
		}))
		.pipe(sourcemaps.write('.'))
		.pipe(gulp.dest(path.join(config.assets, 'js')));
});


/*===================================================================
=            Watch for source changes and rebuild/reload            =
===================================================================*/

gulp.task('watch', function() {
	if (typeof config.server === 'object') {
		gulp.watch([config.dest + '/**/*'], ['livereload']);
	}
	gulp.watch([config.src + '/js/**/*'], ['js']);
	gulp.watch([config.src + '/html/**/*'], ['html']);
	gulp.watch([config.src + '/less/**/*'], ['less']);
	gulp.watch([config.src + '/css/**/*'], ['css']);
	gulp.watch([config.src + '/images/**/*'], ['images']);
	gulp.watch([config.src + '/vendor/**/*'], ['vendor']);
	gulp.watch([config.src + '/fonts/**/*'], ['fonts']);

});


/*===================================================
=            Starts a Weinre Server                 =
===================================================*/

gulp.task('weinre', function() {
	if (typeof config.weinre === 'object') {
		var weinre = require('./node_modules/weinre/lib/weinre');
		weinre.run(config.weinre);
	} else {
		throw new Error('Weinre is not configured');
	}
});


/*======================================
=            Build Sequence            =
======================================*/

gulp.task('build', function(done) {
	var tasks = ['css', 'html', 'vendor', 'fonts', 'images', 'less', 'js'];
	seq('clean', tasks, done);
});

/*====================================
=            Default Task            =
====================================*/

gulp.task('default', function(done) {

	var tasks = [];

	if (typeof config.weinre === 'object') {
		tasks.push('weinre');
	}

	if (typeof config.server === 'object') {
		tasks.push('connect');
	}

	tasks.push('watch');

	seq('build', tasks, done);
});